/*
 * Database.java, 13 août 2005
 * 
 * This file is part of xtnd-commons.
 *
 * Copyright © 2005 Johan Cwiklinski
 *
 * File :               Database.java
 * Author's email :     johan@x-tnd.be
 * Author's Website :   http://www.x-tnd.be/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package be.xtnd.commons.db;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import be.xtnd.commons.gui.MainGui;

/**
 * Connection aux bases de données sql, exécution de requêtes 
 *
 * @author Johan Cwiklinski
 * @version 1.0
 */
public class Database {
	/** Mode ajout */
	public static final char NEW = 'n';
	/** Mode modification */
	public static final char MODIF = 'm';
	private static Connection connection;
	private Statement statement;
	private ResultSet rs;
	private int rsConcurrency = 0;
	private int rsType = 0;
	private int errCode = 0;
	private String provider_name="", sql_server="", sql_user="", sql_password="", sql_provider="";
	private boolean passwordSet;
	static Logger logger = Logger.getLogger(Database.class.getName());
	
	/**
	 * Définit des paramètres de connection additionnels
	 * 
	 * @param type type de connexion (voir types jdbc)
	 * @param ro type de lecture (voir paramètres jdbc)
	 */
	public void setConnectionParameters(int type, int ro){
		this.rsConcurrency = ro;
		this.rsType = type;
	}

	/**
	 * Constructeur par défaut
	 *
	 */
	public Database(){
		this(false);
	}
	
	/**
	 * Constructeur 
	 * 
	 * @param connect Connection automatique ou non. Si connect est à true,
	 * la connection sera établie par le constructeur, s'il est à false, 
	 * il faudra ensuite appeler le méthode connect(). Utile pour passer 
	 * des paramètres particuliers via la méthode setConnectionParameters
	 * 
	 * @see #setConnectionParameters(int, int)
	 * @see #connect()
	 */
	public Database(boolean connect){
		PropertyConfigurator.configure(ClassLoader.getSystemResource("be/xtnd/commons/log4j.properties"));
		if(connect){
			connect();
		}
	}
	
	/**
	 * 
	 * @param provider_name
	 * @param sql_server
	 * @param sql_user
	 * @param sql_password
	 * @param sql_provider
	 */
	public Database(String provider_name, String sql_server, String sql_user,
			String sql_password, String sql_provider){
		this(false);
		this.provider_name = provider_name;
		this.sql_server = sql_server;
		this.sql_user = sql_user;
		setSql_password(sql_password);
		this.sql_provider = sql_provider;
		connect();
	}
	
	/**
	 * @return Renvoie un objet java.sql.Connection.
	 */
	public Connection getConnection() {
		if(connection==null) connect();
		return connection;
	}
	
	/**
	 * Connecte à la base de données soit avec les paramètres fournis via les méthodes adéquates,
	 * soit avec la configuration chargée depui MainGui
	 *
	 * @see be.xtnd.commons.Config	
	 *
	 * @see #setProvider_name(String)
	 * @see #setSql_password(String)
	 * @see #setSql_provider(String)
	 * @see #setSql_server(String)
	 * @see #setSql_user(String)
	 */
	public void connect(){
		if(this.provider_name.equals("")) provider_name = MainGui.conf.getProvider_name();
		if(sql_server.equals("")) sql_server = MainGui.conf.getSql_server();
		if(sql_user.equals("")) sql_user = MainGui.conf.getSql_user();
		if(!this.passwordSet) sql_password = MainGui.conf.getSql_password();
		if(sql_provider.equals("")) sql_provider = MainGui.conf.getSql_provider();
		try{
			logger.debug("*** CONNECTION DATABASE ***");
			logger.debug("*** PROVIDER : "+this.provider_name);
			logger.debug("*** DATABASE : "+this.sql_server);
			logger.debug("*** DRIVER : "+this.sql_provider);
		    Class.forName(this.sql_provider).newInstance();
		    connection = DriverManager.getConnection(
		    		this.sql_server,
		    		this.sql_user,
		    		this.sql_password
					);
		    if(this.rsConcurrency != 0 && this.rsType != 0){
		    	logger.debug("*** CONNECTION ADDITIONAL PARAMETERS type : "+
		    			this.rsType+", concurrency : "+this.rsConcurrency);
		    	statement = connection.createStatement(this.rsType, this.rsConcurrency);
		    }else{
		    	statement = connection.createStatement();
		    }
		    logger.debug("          [Ok]");
	      }catch ( SQLException E){
	    	  logger.fatal("SQLException while trying to connect database : "+E.getMessage()+" (code "+E.getErrorCode()+" -- sqlstate "+E.getSQLState()+") ");
			  logger.debug("          [Failed]");
	          this.errCode = E.getErrorCode();
	          //unknown host exception from mysql
	          //TODO verify with others drivers if the behavior is the same
			  if(E.getSQLState().equals("08S01") && provider_name.equalsIgnoreCase("mysql")){
				this.errCode = -10;  
			  }
		  }
		  catch ( ClassNotFoundException e){
			  logger.fatal("ClassNotFound while trying to connect database : " + e.getMessage());
			  logger.debug("          [Failed]");
		  }
		  catch ( InstantiationException e){
			  logger.fatal("InstantiationException while trying to connect database : " + e.getMessage());
			  logger.debug("          [Failed]");
		  }
		  catch ( IllegalAccessException e){
			  logger.fatal("IllegalAccessException while trying to connect database : " + e.getMessage());
			  logger.debug("          [Failed]");
		  }
		  catch ( NullPointerException e){
			  logger.fatal("NullPointerException while trying to connect database : " + e.getMessage());
			  logger.debug("          [Failed]");
		  }		  
	}
	
	/**
	 * Déconnecte de la base de données
	 *
	 */
	public void disconnect(){
	  	logger.debug("*** DECONNECTION DATABASE ***");
	  	try {
			connection.close();
		  	logger.debug("          [Ok]");
		} catch (SQLException e) {
			logger.fatal("SQLException while trying to disconnect database : "+e.getMessage()+" (code "+e.getErrorCode()+")");
		  	logger.debug("          [Failed]");
		}
	}
	
	/**
	 * Exécute une requête de sélection sql (SELECT)
	 * 
	 * @param strQuery Requête de sélection sql à éxécuter
	 * @return ResultSet
	 * @throws SQLException
	 */
	public ResultSet execQuery(String strQuery) throws SQLException{
	  	logger.debug("*** Execute query "+strQuery+" ***");
        return statement.executeQuery(strQuery);
	}

	/**
	 * Exécute une requête de sélection sql (SELECT)
	 * 
	 * @param strQuery Requête de sélection sql à éxécuter
	 * @param stmt Statement à utiliser pour la requête
	 * @return ResultSet
	 * @throws SQLException
	 */
	public ResultSet execQuery(String strQuery, Statement stmt) throws SQLException{
	  	logger.debug("*** Execute query "+strQuery+" ***");
        return stmt.executeQuery(strQuery);
	}
	
	/**
	 * Exécute une requête sql qui fournit un retour 'UPDATE, INSERT, DELETE, ..)
	 * 
	 * @param strQueryUpdate Requête d'update sql à éxécuter
	 * @return nombre d'enregistrement affectés
	 * @throws SQLException
	 */
	public int execUpdate(String strQueryUpdate) throws SQLException{
        logger.debug("*** Execute update "+strQueryUpdate+" ***");
        return statement.executeUpdate(strQueryUpdate);
	}
	  
	/**
	 * Store une requête pour les listes en table
	 * @param query requête à  exécuter
	 */
	public void setQuery(String query){
		try {rs = statement.executeQuery(query);}
		catch (SQLException E){
			logger.fatal("SQLException while executing query : "+query+" : "+E.getMessage()+" (code "+E.getErrorCode()+")");
		}
	}

	/**
	 * 
	 * @return Renvoie le ResultSet
	 */
	public ResultSet getResultSet(){
		return rs;
	}

	/**
	 * Remplace des chaînes de caractères
	 * @param s : chaîne à modifier
	 * @param f : chaîne à remplacer
	 * @param r : chaîne de remplacement
	 * @return : chaîne modifiée
	 */
	public static String replace( String s, String f, String r ){
	     if (s == null)  return s;
	     if (f == null)  return s;
	     if (r == null)  r = "";

	     int index01 = s.indexOf( f );
	     while (index01 != -1)
	     {
	        s = s.substring(0,index01) + r + s.substring(index01+f.length());
	        index01 += r.length();
	        index01 = s.indexOf( f, index01 );
	     }
	     return s;
	}

	/**
	 * 
	 * @return Renvoie le dernier code d'erreur 
	 */
	public int getErrCode() {
		return errCode;
	}
	
	/**
	 * 
	 * @return Renvoie le nom du provider
	 */
	public String getProvider_name() {
		return provider_name;
	}

	/**
	 * 
	 * @param provider_name Nom du provider à définir
	 */
	public void setProvider_name(String provider_name) {
		this.provider_name = provider_name;
	}

	/**
	 * 
	 * @return Renvoie le mot de passe sql
	 */
	public String getSql_password() {
		return sql_password;
	}

	/**
	 * 
	 * @param sql_password Mot de passe sql à définir
	 */
	public void setSql_password(String sql_password) {
		this.passwordSet = true;
		this.sql_password = sql_password;
	}

	/**
	 * 
	 * @return Renvoie la classe du provider sql
	 */
	public String getSql_provider() {
		return sql_provider;
	}

	/**
	 * 
	 * @param sql_provider Classe du provider sql à définir
	 */
	public void setSql_provider(String sql_provider) {
		this.sql_provider = sql_provider;
	}

	/**
	 * 
	 * @return Renvoie la chaîne de connexion sql
	 */
	public String getSql_server() {
		return sql_server;
	}

	/**
	 * 
	 * @param sql_server Chaîne de connexion slq à définir
	 */
	public void setSql_server(String sql_server) {
		this.sql_server = sql_server;
	}

	/**
	 * 
	 * @return Renvoie le nom d'utilisateur sql
	 */
	public String getSql_user() {
		return sql_user;
	}

	/**
	 * 
	 * @param sql_user Nom d'utilisateur sql à définir
	 */
	public void setSql_user(String sql_user) {
		this.sql_user = sql_user;
	}
}
