/*
 * ResultSetTableModel.java, 31 mai 2005
 * 
 * This file is part of xtnd-commons.
 *
 * Copyright Â© 2005 Johan Cwiklinski
 *
 * File :               ResultSetTableModel.java
 * Author's email :     johan@x-tnd.be
 * Author's Website :   http://www.x-tnd.be/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package be.xtnd.commons.db;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

/**
 * Implémente l'interface <code>TableModel</code> depuis un objet 
 * <code>ResultSet</code>. Une <code>JTable</code> pourra donc 
 * afficher le contenu du <code>ResultSet</code>.
 * Un <code>ResultSet</code> JDBC 2.0 "scrollable" est requis, il sera
 * en lecture seule. 
 * 
 * @author Johan Cwiklinski
 * @version 1.0
 **/
public class ResultSetTableModel implements TableModel {
    ResultSet results;             // ResultSet à interpréter
    ResultSetMetaData metadata;    // Méta données
    int numcols, numrows;          // nombre de lignes et de colonnes
	static Logger logger = Logger.getLogger(ResultSetTableModel.class.getName());


    /**
     * Ce constructeur initialise un <code>TableModel</code> depuis
     * un <code>ResutSet</code>
     * Utilisé par @link ResultSetTableModelFactory qu'il faut 
     * utiliser pour obtenir le modèle généré. 
     * 
     * @param results <code>ResultSet</code>
     * @throws SQLException
     **/
    ResultSetTableModel(ResultSet results) throws SQLException {
    	PropertyConfigurator.configure(ClassLoader.getSystemResource("be/xtnd/commons/log4j.properties"));
    	this.results = results;                 // enregistre le ResultSet
    	metadata = results.getMetaData();       // récupère les méta datas
    	numcols = metadata.getColumnCount();    // combien de colonnes ?
    	results.last();
    	numrows = results.getRow();             // combien de lignes ?
    }
    
    /** 
     * Apellée quand le <code>TableModel</code> est détruit.
     * Ferme le <code>Statement</code> et le <code>ResultSet</code>
     **/
    public void close() {
    	try { results.getStatement().close(); }
    	catch(SQLException e) {};
    }

    /** Fermeture automatique sur garbage collector */
    protected void finalize() { close(); }

    /** 
     * Renvoie le nombre de colonnes
     * 
     * @return int colonnes nombre de colonnes
     */
    public int getColumnCount() { return numcols; }

    /**
     * Renvoie le nombre de lignes
     * 
     * @return int lignes nombre de lignes
     */
    public int getRowCount() { return numrows; }

    /**
     * Retourne le nom de la colonne selon 
     * les méta données du <code>ResultSet</code>
     * 
     * @param column numéro de colonne
     * 
     * @return String nom de la colonne 
     */
    public String getColumnName(int column) {
	try {
	    return metadata.getColumnLabel(column+1);
	} catch (SQLException e) { return e.toString(); }
    }

    /**
     * Renvoie la classe de la colonne
     * 
     * @param column colonne à étudier
     * 
     * @return classe de la colonne
     */
    public Class<?> getColumnClass(int column) {
    	return (getValueAt(0, column)==null)?String.class:getValueAt(0, column).getClass();
    }
    
    /**
     * Retourne la valeur de la cellule.
     * Attention : les numéros des lignes colonnes du <code>ResultSet</code> débutent à 1,
     * celles des <code>TableModel</code> à 0 !
     * 
     * @param row ligne à évaluer
     * @param column colonne à évaluer
     * 
     * @return Object valeur de la cellule évaluée
     **/
    public Object getValueAt(int row, int column) {
    	try {
    		results.absolute(row+1);
    		Object o = results.getObject(column+1); 
    		if (o == null) return null;       
    		else return o;
    	} catch (SQLException e) {
    		logger.error("SQLException in RSTableModel : "+e.getMessage());
    		return null;
		}
    }
    
    /**
     * Trouve une valeur textuelle et renvoie le numéro de ligne associée
     * @param s chaîne à chercher
     * @param col dans quelle colonne chercher
     * @return ligne trouvée
     */
    public int findString(String s, int col){
    	int anInt = -1;
    	try{
    		results.first();
	    	while(results.next()){
	    		//System.out.println(results.getObject(col+1));
	    		if(results.getObject(col+1).toString().equals(s))
	    			return results.getRow();
	    	}
    	}catch(SQLException e){
    		logger.error("SQLException finding text value in RSTableModel : "+e.getMessage());    		
    	}
    	return anInt;
    }

    /**
     * Table non editable par défaut
     * @param row ligne à évaluer
     * @param column colonne à évaluer
     * @return boolean true si la cellule évaluée est éditable, false sinon. Renvoie toujours false.
     */
    public boolean isCellEditable(int row, int column) { return false; } 
    // méthodes qu'il n'est pas nécéssaire d'implémenter, la table étant non éditable
    /**
     * @param value valeur à enregistrer
     * @param row ligne à traiter
     * @param column colonne à traiter
     */
    public void setValueAt(Object value, int row, int column) {}
    /**
     * @param l TableModelListener
     */
    public void addTableModelListener(TableModelListener l) {}
    /**
     * @param l TableModelListener
     */
    public void removeTableModelListener(TableModelListener l) {}
}
