/*
 * MaskFieldVerifier.java, 21 sept. 2005
 * 
 * This file is part of xtnd-commons.
 *
 * Copyright © 2005 Johan Cwiklinski
 *
 * File :               MaskFieldVerifier.java
 * Author's email :     johan@x-tnd.be
 * Author's Website :   http://www.x-tnd.be/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package be.xtnd.validators;

import java.text.MessageFormat;
import java.text.ParseException;
import java.util.ResourceBundle;

import javax.swing.InputVerifier;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JOptionPane;
import javax.swing.JFormattedTextField.AbstractFormatter;

import be.xtnd.commons.MessagesCommons;
import be.xtnd.commons.gui.MainGui;


/**
 * Vérifie que la valeur en entrée correspond au masque donné.
 * Attention toutefois, cette classe ne semble fonctionner qu'avec des
 * MaskFormatter qui ne comportent pas de caractère littéraux inutiles
 * hormis les espaces (ISBN d'un livre par ex. : #-###-#####-#).
 * Dans ce cas, la valeur du texte si le FTF est vide sera toujours
 * composée des espaces et des tirets.
 *
 * @author Johan Cwiklinski
 * @version 1.0
 */
public class MaskFieldVerifier extends InputVerifier {
    private String the_value, mask_value;
    private ResourceBundle bundle;
    private String bundle_key = "window.messages.error.mask";
    
    /**
     * Constructeur par défaut. Cela initialisera la vérification de
     * la valeur entrée en paramètre. Le message affiché en cas d'erreur
     * correspondra à l'entrée window.messages.error.mask
     * 
     * @param the_value la chaîne à vérifier
     * @param mask_value le masque de vérification
     * @param bundle ResourceBundle à lire
     */
	public MaskFieldVerifier(String the_value, String mask_value, ResourceBundle bundle) {
		this.the_value = the_value;
		this.mask_value = mask_value;
		this.bundle = bundle;
	}
	
    /**
     * Constructeur par défaut. Cela initialisera la vérification de
     * la valeur entrée en paramètre. Le message affiché en cas d'erreur
     * correspondra à l'entrée du paramètre bundle_key
     * 
     * @param the_value la chaîne à vérifier
     * @param mask_value le masque de vérification
     * @param bundle ResourceBundle à lire
     * @param bundle_key la clé bundle à afficher en cas d'erreur
     */	
	public MaskFieldVerifier(String the_value, String mask_value, ResourceBundle bundle, 
			String bundle_key){
		this.the_value = the_value;
		this.mask_value = mask_value;
		this.bundle = bundle;
		this.bundle_key = bundle_key;
	}
	
	/**
	 * Renvoie le résultat de la vérification du contenu par rapport au masque.
	 * 
	 * @param input le composant en entrée
	 * @return true s'il y a correspondance, false sinon
	 */
	@Override
	public boolean verify(JComponent input) {
        if (input instanceof JFormattedTextField) {
            JFormattedTextField ftf = (JFormattedTextField)input;
            AbstractFormatter formatter = ftf.getFormatter();
            if (formatter != null) {
                String text = ftf.getText();
                if(!text.trim().equals("")){
	                 try {
	                      formatter.stringToValue(text);
	                      return true;
	                  } catch (ParseException pe) {
	                  		Object[] args = {mask_value};
	                  		int response = JOptionPane.showConfirmDialog(
	                  				MainGui.desktop, 
									MessageFormat.format(bundle.getString(bundle_key), args),
									MessagesCommons.getString("window.messages.error"),
									JOptionPane.YES_NO_OPTION,
									JOptionPane.WARNING_MESSAGE);
	                  		if(response==JOptionPane.NO_OPTION){
	                  			if(the_value.trim().equals("")){
	                  				ftf.setText(null);
	                  				ftf.setValue(null);
	                  			}else if(!the_value.trim().equals("")){
	                  				ftf.setText(the_value);
	                  			}else if(ftf.getValue()!=null){
	                  				ftf.setText(ftf.getValue().toString());
	                  			} 
	                  			return true;
	                  		}
	                  		return false;
	                  }
                }else{
                	ftf.setValue(null);
                }
             }
         }
         return true;
     }
	
	/**
	 * Le composant doit-il céder le focus.
	 * 
	 * @param input le composant en entrée
	 * @return true si le texte correspond au masque, false sinon
	 */
	@Override
    public boolean shouldYieldFocus(JComponent input) {
         return verify(input);
    }
}